////////////////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 2007 IWantGamez, LLC.  All rights reserved.
//  
//  Author: John Holben         Date: 08.28.2007
//
////////////////////////////////////////////////////////////////////////////////

// Constants 
var LOOP_INTERVAL = 50; // 20 fps
var GS_IDLE = 0;
var GS_PLAY_INTRO = 1;
var GS_INIT_PLAY_SEQUENCE = 2;
var GS_PLAY_SEQUENCE = 3;
var GS_PLAY_PAUSE = 4;
var GS_BLANK_PAUSE = 5
var GS_WAIT_FOR_INPUT = 6;
var GS_FINISHED = 7;
var GS_GAME_OVER = 8;
var GS_END_GAME_OVER = 9;
var GS_SHOW_MESSAGE = 10;
var GS_SHOW_WATCH_MESSAGE = 11;
var GS_FINISHED_LEVEL = 13;
var GS_GAME_OVER_BLINK_ON = 14;
var GS_GAME_OVER_BLINK_OFF = 15;

var MAX_LEVEL = 99;

var YELLOW = 0;
var RED = 1;
var BLUE = 2;
var GREEN = 3;
var ALL_ON = 4;
var ALL_OFF = 5;


var TRANSITION_TIME = 1;
var GADGET_WIDTH = 130;
var GADGET_HEIGHT = 130;
var SCALE_DOCKED = 1;
var SCALE_UNDOCKED = 2;

// Audio
var WAV_RED = 0;
var WAV_GREEN = 1;
var WAV_BLUE = 2;
var WAV_YELLOW = 3;
var WAV_GAME_OVER = 4;
var WAV_GAME_START = 5;
var WAV_GAME_QUIT = 6;
var WAV_HI_SCORE = 7;

// From settings.js
var KEY_ENABLE_SOUND_FX = "SoundFX";
var KEY_HS_MOVES = "HSMoves";
var KEY_EXISTS = "EXISTS";

// Globals
var AUDIO = new AudioTemplate();
var TOTAL_SECONDS = 0;
var GAME_LOOP;
var GAME_OVER = true;
var QUIT = false;
var LIGHTS;
var SEQUENCE = new Array(MAX_LEVEL);
var LEVEL = 0;

// Debug
var DEBUG = 0;

function Initialize()
{
    // Create shortcut
     LIGHTS = new LightsTemplate();
    // Get audio going 
    AUDIO.Init();
    // Monitor incase player turns off soundFx
    System.Gadget.onSettingsClosed = AUDIO.SettingsClosed;
    System.Gadget.settingsUI = "settings.html";
    // Monitor Dock state to display proper graphics
    System.Gadget.onDock = HandleDockChanged
    System.Gadget.onUndock = HandleDockChanged
    HandleDockChanged();
    // Create the Game Loop 
    GAME_LOOP = new GameLoopTemplate();
    // Initialize UI state 
    ConfigureMenus();
    // This is required so control is focused and Tool tips will display
    // before any clicking of the game pieces
    self.focus();
    document.body.focus();
}

function HandleDockChanged()
{
    // Grab short-cuts to controls
    var actionButton = document.getElementById("actionButton");
    var helpButton = document.getElementById("helpButton");
    var replayButton = document.getElementById("replayButton");
    var top10Text = document.getElementById("top10Text");
    var level = document.getElementById("level");
    var messageDiv = document.getElementById("message");
    var halt = document.getElementById("halt");
    var bodyStyle = document.body.style;
    var newWidth;
    var newHeight;
    
    //System.Gadget.beginTransition();
    
    if(System.Gadget.docked)
    {
        newWidth = GADGET_WIDTH * SCALE_DOCKED;
        newHeight = GADGET_HEIGHT * SCALE_DOCKED;

        halt.src = "Graphics/Docked/halt.png";
        top10Text.style.fontSize = 9;
        top10Text.style.left = 48;
        top10Text.style.top = 52;
        
        actionButton.style.top = 67; 
        actionButton.style.left = 51; 
        actionButton.style.width = 12;
        actionButton.style.height = 12;

        helpButton.style.top = 67;
        helpButton.style.left = 51 + 14;
        helpButton.style.width = 12;
        helpButton.style.height = 12;
        
        replayButton.style.top = 67;
        replayButton.style.left = 47 + 16;
        replayButton.style.width = 13;
        replayButton.style.height = 14;
        

        level.style.top = 47;
        level.style.left = 56;
        level.style.fontSize = 12;
        
        
        messageDiv.style.left = 5;
        messageDiv.style.top = 16;
        
        
    }
    else
    {
        newWidth = GADGET_WIDTH * SCALE_UNDOCKED;
        newHeight = GADGET_HEIGHT * SCALE_UNDOCKED;

        top10Text.style.fontSize = 14;
        top10Text.style.left = 104;
        top10Text.style.top = 108;

        actionButton.style.left = (newWidth / 2) - 29; 
        actionButton.style.top = 135; 
        actionButton.style.width = 16;
        actionButton.style.height = 16;

        level.style.top = 96;
        level.style.left = 112;
        level.style.fontSize = 20;

        replayButton.style.left = (newWidth / 2) + 7;
        replayButton.style.top = 135;
        replayButton.style.width = 16;
        replayButton.style.height = 16;

        helpButton.style.left = (newWidth / 2) + 7;
        helpButton.style.top = 135;
        helpButton.style.width = 16;
        helpButton.style.height = 16;

        messageDiv.style.left = 70;
        messageDiv.style.top = 70;

        halt.src = "Graphics/Undocked/halt.png";
    }

    bodyStyle.width = newWidth;
    bodyStyle.height = newHeight;

    LIGHTS.SetSize(newWidth,newHeight);
    LIGHTS.ConfigureMap();
    LIGHTS.SetLightState(ALL_OFF);

    //System.Gadget.endTransition(System.Gadget.TransitionType.morph, TRANSITION_TIME);   
}

function ShowHelp()
{
    System.Gadget.Flyout.file = 'help.html'; 
    System.Gadget.Flyout.show = true;    
}

function GreenClick()
{
    LIGHTS.PlayerClick(GREEN);
}

function BlueClick()
{
    LIGHTS.PlayerClick(BLUE);
}

function RedClick()
{
    LIGHTS.PlayerClick(RED);
}

function YellowClick()
{
    LIGHTS.PlayerClick(YELLOW);
}

function ConfigureMenus()
{
    var actionButton = document.getElementById("actionButton");
    var helpButton = document.getElementById("helpButton");
    var top10Text = document.getElementById("top10Text");
    var level = document.getElementById("level");

    if(GAME_OVER == true)
    {
        actionButton.alt = "Play";
        actionButton.src = "../Graphics/PlayButton.png";
        actionButton.style.visibility = "visible";
        top10Text.style.visibility = "visible";
        level.style.visibility = "hidden";           
        helpButton.style.visibility = "visible";
    
    }
    else
    {
        actionButton.alt = "Quit";
        actionButton.src = "../Graphics/QuitButton.png";
        top10Text.style.visibility = "hidden";
        level.style.visibility = "visible";           
        helpButton.style.visibility = "hidden";
    }
}

function ActionClick()
{
    if(GAME_OVER == true)
    {
        QUIT = false;
        GAME_OVER = false;
        LIGHTS.ConfigureMap();        
        ConfigureMenus();
        LEVEL = 0;
        UpdateSequence();
        LIGHTS.Initialize();
    
        AUDIO.Play(WAV_GAME_START);
        // Start the Game Loop running as fast as possible
        setTimeout(GAME_LOOP.Run,1);
    }
    else
    {
        // Player just quit
        QUIT = true;
        GAME_OVER = true;
        LIGHTS.ConfigureMap(); 
        AUDIO.Play(WAV_GAME_QUIT);
    }
}

function ActionMouseOver(actionButton)
{
    if(GAME_OVER == true)
        actionButton.src = "../Graphics/PlayButtonHover.png";
    else
        actionButton.src = "../Graphics/QuitButtonHover.png";
}

function ActionMouseOut(actionButton)
{
    if(GAME_OVER == true)
        actionButton.src = "../Graphics/PlayButton.png";
    else
        actionButton.src = "../Graphics/QuitButton.png";
}

function showTop10()
{
    System.Gadget.Flyout.file = 'top10.html'; 
    System.Gadget.Flyout.show = true;    
}    

function GameOver()
{
    var actionButton = document.getElementById("actionButton");
    var helpButton = document.getElementById("helpButton");
    var level = document.getElementById("level");
    var lights = document.getElementById("lights");
    
    actionButton.style.visibility = "hidden";
    helpButton.style.visibility = "hidden";
    level.style.visibility = "hidden";

    var count = 1;
    if(System.Gadget.Settings.read(KEY_EXISTS) == true)
    {
        //debugger;
        count = parseInt(System.Gadget.Settings.readString(KEY_HS_MOVES),10);
        if(isNaN(count) == true)
            count = 1;
    }   
    
    if((LIGHTS.count - 1) > count && QUIT == false)
    {
        // Save new hi score
        System.Gadget.Settings.writeString(KEY_HS_MOVES,LIGHTS.count - 1);
        System.Gadget.Settings.write(KEY_ENABLE_SOUND_FX,AUDIO.UseSoundFx);
        System.Gadget.Settings.write(KEY_EXISTS, true);
        AUDIO.Play(WAV_HI_SCORE); 
        lights.filters.item(0).Apply();
        lights.src = "../Graphics/trophie.png";
        lights.filters.item(0).Play();
    }
    else
    {
        LIGHTS.Uninitialize();
        ConfigureMenus();
    }
}

function FadeFinished()
{
    var lights = document.getElementById("lights");

    if(GAME_OVER == true)
    {
        GAME_OVER = false;
        setTimeout(FinishGameOver,2000);
    }
    else
    {
        GAME_OVER = true;
        LIGHTS.Uninitialize();
        ConfigureMenus();
    }
}

function FinishGameOver()
{
    lights.filters.item(0).Apply();
    LIGHTS.SetLightState(ALL_OFF);
    lights.filters.item(0).Play();
}

function UpdateSequence()
{
    LEVEL++;
    
    if(LEVEL > MAX_LEVEL)
    {
        GAME_OVER = true;    
        return;
    }
    
    var nextSequence;
    if(LEVEL > 2)
    {
        for(i =0;i<9;i++)
        {
            nextSequence = Math.round(Math.random() * 3);
            
            // Don't let three in a row go
            if(nextSequence == SEQUENCE[LEVEL -2] && nextSequence == SEQUENCE[LEVEL - 3])
            {
                continue;
            }
            else
                break;
        }
    }
    else
    {
        nextSequence = Math.round(Math.random() * 3);
    }
    
    SEQUENCE[LEVEL-1] = nextSequence;
}

function GameLoopTemplate()
{
    this.lastDate = new Date();
    this.totalTicks = 0;
    this.subTick = 0;
    this.tempGems = 0;
    this.state = 0;
    this.light = 0;

    this.Run = function() 
        {
            if(DEBUG == 1) { debugger; DEBUG = 0; }
            
            var now = new Date();
            var ticks = now - GAME_LOOP.lastDate;
            GAME_LOOP.lastDate = now;
            GAME_LOOP.totalTicks += ticks;
            GAME_LOOP.subTick += ticks;
            
            if(GAME_LOOP.subTick > 1000)
            {
                var seconds = parseInt(GAME_LOOP.subTick / 1000);
                GAME_LOOP.subTick %= 1000;
                
                if(GAME_OVER == false)
                    TOTAL_SECONDS += seconds;    
            }
            
            // Update Status Display
            var timeCount = Math.abs(TOTAL_SECONDS);
            var hours = parseInt(timeCount / 3600);
            var minutes = parseInt((timeCount / 60) % 60);
            var seconds = parseInt(timeCount % 60);
            //timerCount.innerText = (hours < 10 ? "0" : "")+hours+":"+(minutes < 10 ? "0" : "")+minutes+":"+(seconds < 10 ? "0" : "")+seconds;
            
            var timerAccuracy = ticks / LOOP_INTERVAL;
            
            
            LIGHTS.Tick(GAME_LOOP.totalTicks);
            
            
            if(GAME_OVER == false)
            {
                // Restart Loop
                setTimeout(function() {GAME_LOOP.Run();}, LOOP_INTERVAL);
            }
            else
            {
                GameOver();
            }
        }

    this.Tick = function()
        {
            LIGHTS.SetLightState(this.state);
        }
}

function LightsTemplate()
{
    this.lights = document.getElementById("lights");
    this.replayButton = document.getElementById("replayButton");
    this.messageDiv = document.getElementById("message");
    this.levelDiv = document.getElementById("level");
    this.halt = document.getElementById("halt");
    this.showReplay = false;
    this.state = GS_IDLE;
    this.count = 0;
    this.lastTick = 0;
    this.tempLastTick = 0;
    this.pause = 400;
    this.playerIndex = 0;
    this.nextState = 0;
    this.lightOnCount = 0;
    this.blink = 0;
    
    this.Initialize = function()
    {
        this.state = GS_SHOW_WATCH_MESSAGE;
        this.count = 0;
        this.pause = 400;
        this.lastTick = -1;
        this.tempLastTick = 0;
        this.playerIndex = 0;
        this.lightOnCount = 0;
        this.showReplay = true;
        var temp = "00" + LEVEL;
        this.levelDiv.innerText = temp.substr(temp.length - 2);
    }
    
    this.Uninitialize = function()
    {
        // Game Over Clean Up 
        this.SetLightState(ALL_OFF);
        this.ConfigureMap();
        this.halt.style.visibility = "hidden";
        this.levelDiv.style.visibility = "hidden";
        this.replayButton.style.visibility = "hidden";
        this.messageDiv.style.visibility = "hidden";
    
    }
    
    this.Tick = function(ticks)
    {
        if(this.lastTick == -1)
            this.lastTick = ticks;
            
        if(this.lightOnCount > 0)
        {
            this.lightOnCount--;
            
            if(this.lightOnCount == 0)
                this.SetLightState(ALL_OFF);
        }    
            
        this.tempLastTick = ticks;
        if(DEBUG == 1)
        {
            DEBUG = 0;
            debugger;
        }
    
        switch(this.state)
        {
            case GS_SHOW_WATCH_MESSAGE:
            
                this.lastTick = ticks;
                this.state = GS_SHOW_MESSAGE;
                this.nextState = GS_INIT_PLAY_SEQUENCE;            
                this.halt.style.visibility = "visible";
                break;

            case GS_SHOW_MESSAGE:
                
                if(ticks - this.lastTick > 1000)
                {
                    this.lastTick = ticks;
                    this.messageDiv.style.visibility = "hidden";
                    
                    if(this.nextState == GS_SHOW_WATCH_MESSAGE)
                    {
                        var temp = "00" + LEVEL;
                        this.levelDiv.innerText = temp.substr(temp.length - 2);
                    }
                    this.state = this.nextState;
                }
                break;
                
            case GS_BLANK_PAUSE:
            
                if(ticks - this.lastTick > 500)
                {
                    this.lastTick = ticks;
                    var temp = "00" + LEVEL;
                    this.levelDiv.innerText = temp.substr(temp.length - 2);
                    this.state = this.nextState;
                }
                break;

            case GS_INIT_PLAY_SEQUENCE:

                this.count = 0;
                this.SetLightState(SEQUENCE[this.count]);
                this.PlaySound(SEQUENCE[this.count]);
                this.state = GS_PLAY_PAUSE;    
                this.lastTick = ticks;
                break;
                
            case GS_PLAY_PAUSE:

                if(ticks - this.lastTick > this.pause)
                {
                    this.lastTick = ticks;
                    this.SetLightState(ALL_OFF);
                    this.state = GS_PLAY_SEQUENCE;                      
                }
                break;    
                
            case GS_PLAY_SEQUENCE:
                
                if(ticks - this.lastTick > this.pause)
                {
                    this.lastTick = ticks;
                    this.count++;
                    
                    if(this.count < LEVEL)
                    {   
                        this.SetLightState(SEQUENCE[this.count]);
                        this.PlaySound(SEQUENCE[this.count]);
                        this.state = GS_PLAY_PAUSE;    
                    }
                    else
                    {
                        this.SetLightState(ALL_OFF);
                        this.playerIndex = 0;
                        this.state = GS_WAIT_FOR_INPUT;
                        if(this.showReplay == true)
                            this.replayButton.style.visibility = "visible";
                        this.halt.style.visibility = "hidden";
                    }                    

                }
                break;    

            case GS_WAIT_FOR_INPUT:
                
                if(ticks - this.lastTick > 10000)
                {
                    this.state = GS_GAME_OVER_BLINK_ON;
                    this.SetLightState(ALL_ON);
                    this.blink = 0;
                    this.messageDiv.innerText = "Game Over!";
                    this.messageDiv.style.visibility = "visible";
                    AUDIO.Play(WAV_GAME_OVER);    
                    QUIT = true;
                }
                break;                    
            
            case GS_FINISHED_LEVEL:

                UpdateSequence();
                this.pause -= 20;
                
                if(this.pause < 200)
                    this.pause = 200;
                    
                this.state = GS_SHOW_MESSAGE;
                this.messageDiv.innerText = "Perfect!";
                this.messageDiv.style.visibility = "visible";
                this.nextState = GS_SHOW_WATCH_MESSAGE;
                this.showReplay = true;
                break;
                
            case GS_GAME_OVER_BLINK_ON:
            
                if(ticks - this.lastTick > 200)
                {
                    this.lastTick = ticks;
                    this.SetLightState(ALL_OFF);
                    this.state = GS_GAME_OVER_BLINK_OFF;
                }            
                break;
                
            case GS_GAME_OVER_BLINK_OFF:
            
                if(ticks - this.lastTick > 200)
                {
                    this.lastTick = ticks;
                    this.blink++;
                    
                    if(this.blink > 4)
                    {
                        this.messageDiv.style.visibility = "hidden";
                        this.state = GS_GAME_OVER;
                        QUIT = false;
                    }
                    else
                    {
                        this.SetLightState(ALL_ON);
                        this.state = GS_GAME_OVER_BLINK_ON;
                    }
                }            
                break;
            
            case GS_GAME_OVER:

                GAME_OVER = true;
                break;
                                
            default:
                break;
        }
    
    }
    
    this.PlayerClick = function(color)
    {
        if(this.state != GS_WAIT_FOR_INPUT || GAME_OVER == true)
            return;
    
        this.SetLightState(color);
        this.PlaySound(color);
        this.lightOnCount = 3;
    
        // Hide Replay as soon as player starts clicking
        this.replayButton.style.visibility = "hidden";

        if(color == SEQUENCE[this.playerIndex])
        {
            this.playerIndex++;
            if(this.playerIndex == LEVEL)
            {
                this.state = GS_BLANK_PAUSE;
                this.nextState = GS_FINISHED_LEVEL;
                this.lastTick = -1;
            }
            else
            {
                // Reset the player input timeout
                this.lastTick = this.tempLastTick;
            }
        }
        else
        {
            // Game Over!!        
            this.state = GS_GAME_OVER_BLINK_ON;
            this.SetLightState(ALL_ON);
            this.blink = 0;
            this.messageDiv.innerText = "Game Over!";
            this.messageDiv.style.visibility = "visible";
            AUDIO.Play(WAV_GAME_OVER);    
            this.lastTick = -1;
        }
    }
    
    this.PlaySound = function(color)
    {
        switch(color)
        {
            case RED:
                AUDIO.Play(WAV_RED);
                break;
            case GREEN:
                AUDIO.Play(WAV_GREEN);
                break;
            case BLUE:
                AUDIO.Play(WAV_BLUE);
                break;
            case YELLOW:
                AUDIO.Play(WAV_YELLOW);
                break;
        }
    
    }
    
    
    this.Replay = function() 
        { 
            this.showReplay = false;
            this.replayButton.style.visibility = "hidden";
            this.state = GS_SHOW_WATCH_MESSAGE;  
            
        } 
    
    this.ConfigureMap = function()
    {
        if(GAME_OVER == true)
        {
            this.lights.useMap = "";
        }        
        else
        {
            if(System.Gadget.docked == true)
                this.lights.useMap = "#DockedMap";
            else
                this.lights.useMap = "#UndockedMap";
         }       
        
    }
    
    this.SetLightState = function(state)
    {
        var path;
        var graphic;
        
        if(System.Gadget.docked == true)
        {
            path = "Graphics/Docked/";
        }
        else
        {
            path = "Graphics/Undocked/";        
        }
        
            
        switch(state)
        {
            case ALL_OFF:
                graphic = path + "AllOff.png";
                break;
            case ALL_ON:
                graphic =  path + "AllOn.png";
                break;
            case RED:
                graphic = path + "RedOn.png";
                break
            case GREEN:
                graphic = path + "GreenOn.png";
                break;
            case BLUE:
                graphic = path + "BlueOn.png";
                break;
            case YELLOW:
                graphic =  path + "YellowOn.png";
                break;
            default:
                graphic = path + "AllOff.png";
                break;
        }
        
        this.lights.src = graphic;    
    }

    this.SetSize = function(width,height) { this.lights.width = width; this.lights.height = height; }

}


function AudioTemplate()
{
    this.UseSoundFx = true;

    this.Waves = new Array( "LowC.wav", "G.wav", "C.wav", "E.wav", "YouLose.wav", "gameStart.wav", "gameQuit.wav", "hiScore.wav","echo.wav");
    
    this.Init = function()
        {
            if(System.Gadget.Settings.read(KEY_EXISTS) == true)
            {
                this.UseSoundFx = System.Gadget.Settings.read(KEY_ENABLE_SOUND_FX);        
            }
        }
    
    this.Play = function(id)
        {
            if(this.UseSoundFx == true && id >= 0 && id < this.Waves.length)
            {
                System.Sound.playSound("Waves\\" + this.Waves[id]);            
            }
        }
        
        
    this.SettingsClosed = function(event)
        {
        
            if (event.closeAction == event.Action.commit)
            {
                AUDIO.UseSoundFx = System.Gadget.Settings.read(KEY_ENABLE_SOUND_FX); 
                
            }
        }
}

// For Debugging only 
function KeyHandler()
{
    // Press F5 to attach debugger 
    if (event.keyCode == 116)
    {
        //DEBUG = 1;
    }
}